import 'package:collection/collection.dart';
import 'package:fuzzywuzzy/fuzzywuzzy.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'package:intl/intl.dart';
import 'package:regular_expressions_bot/weather/cities_data.dart';
import 'package:regular_expressions_bot/weather/keywords.dart';
import 'package:weather/weather.dart';

enum AvailableLanguage {
  en,
  pl;

  String get locale => switch (this) {
        AvailableLanguage.en => 'en_US',
        AvailableLanguage.pl => 'pl_PL',
      };

  Language get weatherLocale => switch (this) {
        AvailableLanguage.en => Language.ENGLISH,
        AvailableLanguage.pl => Language.POLISH,
      };
}

final class WeatherController {
  WeatherController() {
    initializeDateFormatting('en_US');
    initializeDateFormatting('pl_PL');
  }

  late AvailableLanguage chooseLanguge;
  late DateTime chooseDate;
  late String chooseCity;

  void setEnglishLanguage() => chooseLanguge = AvailableLanguage.en;

  void setPolishLanguage() => chooseLanguge = AvailableLanguage.pl;

  Future<Weather?> getWeather() async {
    final weatherFactory = WeatherFactory(
      'e8886278bb545d862c00ff619d3f986f',
      language: chooseLanguge.weatherLocale,
    );

    final weatherList =
        await weatherFactory.fiveDayForecastByCityName(chooseCity);

    return weatherList.firstWhereOrNull((e) => e.date!.day == chooseDate.day);
  }

  bool detectDateTimeInEnglish(String message) {
    final parsedNumbers = _extractDate(message);

    if (parsedNumbers != null) {
      chooseDate = parsedNumbers;
      return true;
    }

    for (final keyWord in WeatherKeywords.englishTodayKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now();
        return true;
      }
    }

    for (final keyWord in WeatherKeywords.englishYesterdayKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now().subtract(Duration(days: 1));
        return true;
      }
    }

    for (final keyWord in WeatherKeywords.englishTomorrowKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now().add(Duration(days: 1));
        return true;
      }
    }

    return false;
  }

  bool detectCityInEnglish(String message) {
    for (final city in mainCitiesEnglish) {
      if (partialRatio(city, message) >= 70) {
        chooseCity = city;
        return true;
      }
    }

    return false;
  }

  bool detectDateTimeInPolish(String message) {
    final parsedNumbers = _extractDate(message);

    if (parsedNumbers != null) {
      chooseDate = parsedNumbers;
      return true;
    }

    for (final keyWord in WeatherKeywords.polishTodayKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now();
        return true;
      }
    }
    for (final keyWord in WeatherKeywords.polishYesterdayKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now().subtract(Duration(days: 1));
        return true;
      }
    }
    for (final keyWord in WeatherKeywords.polishTomorrowKeywords) {
      if (partialRatio(keyWord, message) >= 50) {
        chooseDate = DateTime.now().add(Duration(days: 1));
        return true;
      }
    }

    return false;
  }

  bool detectCityInPolish(String message) {
    for (final city in mainCitiesPolish) {
      if (partialRatio(city, message) >= 70) {
        chooseCity = city;
        return true;
      }
    }

    return false;
  }

  static RegExp get weatherKeywordInEnglish => RegExp(
        WeatherKeywords.englishWeatherKeywords.join('|'),
        caseSensitive: false,
      );

  static RegExp get weatherKeywordInPolish => RegExp(
        WeatherKeywords.polishWeatherKeywords.join('|'),
        caseSensitive: false,
      );

  DateTime? _extractDate(String message) {
    final locale = chooseLanguge.locale;

    final regexDateFormatMap = <RegExp, DateFormat>{
      RegExp(r'\b(\d{1,2}/\d{1,2}/\d{2,4})\b'): DateFormat('d/M/y', locale),
      RegExp(r'\b(\d{1,2}/\d{1,2}/\d{2})\b'): DateFormat('MM/dd/yy', locale),
      RegExp(r'\b(\d{2,4}-\d{1,2}-\d{1,2})\b'): DateFormat('y-M-d', locale),
      RegExp(r'\b(\d{1,2}\.\d{1,2}\.\d{2,4})\b'):
          DateFormat('dd.MM.yyyy', locale),
      RegExp(
        r'\b(\d{2,4} [a-zA-Z]+ \d{1,2})\b',
      ): DateFormat('y MMMM d', locale),
      RegExp(r'\b(\d{1,2} [a-zA-Z]+ \d{2,4})\b'):
          DateFormat('d MMMM y', locale),
      RegExp(r'\b(\d{1,2} \d{1,2} \d{2,4})\b'): DateFormat('d MM y', locale),
      RegExp(r'\b(\d{2,4} \d{1,2} \d{1,2})\b'): DateFormat('y MM d', locale),
      RegExp(r'\b(\d{4}\.\d{1,2}\.\d{1,2})\b'): DateFormat('yyyy.MM.dd'),
      RegExp(r'\b(\d{1,2}-[a-zA-Z]+-\d{2,4})\b'): DateFormat('d-MMM-y', locale),
      RegExp(r'\b(\d{1,2}/\d{1,2}/\d{2,4})\b'): DateFormat('M/d/yyyy', locale),
      RegExp(r'\b(\d{1,2}/\d{1,2})\b'): DateFormat('M/d', locale),
      RegExp(r'\b(\d{1,2} \d{1,2})\b'): DateFormat('d MM', locale),
      RegExp(r'\b(\d{1,2} \d{1,2})\b'): DateFormat('MM d', locale),
      RegExp(r'\b(\d{1,2}-[a-zA-Z]+)\b'): DateFormat('d-MMM', locale),
      RegExp(r'\b([a-zA-Z]+ \d{1,2})\b'): DateFormat('MMMM d', locale),
      RegExp(r'\b(\d{1,2} [a-zA-Z]+)\b'): DateFormat('d MMMM', locale),
      RegExp(r'\b(\d{1,2}\.\d{1,2})\b'): DateFormat('dd.MM', locale),
    };

    DateTime? parsedDate;

    regexDateFormatMap.forEach((key, value) {
      final match = key.firstMatch(message);

      if (match != null) {
        final matchedString = match.group(0)!;
        final parsedDateFromString = value.tryParse(matchedString);

        if (parsedDateFromString != null) {
          parsedDate = parsedDateFromString;
          return;
        }
      }
    });

    return parsedDate;
  }
}
